@@ -10,6 +10,7 @@
 #               2) Joachim Loehr private communication
 #               3) http://homepage3.nifty.com/kamisaka/makernote/makernote_casio.htm
 #               4) http://www.gvsoft.homedns.org/exif/makernote-casio.html
+#               JD) Jens Duttke private communication
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Casio;
@@ -18,8 +19,9 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.19';
+$VERSION = '1.28';
 
+# older Casio maker notes (ref 1)
 %Image::ExifTool::Casio::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
@@ -59,7 +61,7 @@ $VERSION = '1.19';
     0x0004 => [
         {
             Name => 'FlashMode',
-            Condition => '$self->{CameraModel} =~ /^QV-(3500EX|8000SX)/',
+            Condition => '$self->{Model} =~ /^QV-(3500EX|8000SX)/',
             Writable => 'int16u',
             PrintConv => {
                 1 => 'Auto',
@@ -162,18 +164,37 @@ $VERSION = '1.19';
         },
     },
     0x0014 => {
-        Name => 'CCDSensitivity',
+        Name => 'ISO',
         Writable => 'int16u',
-        PrintConv => {
-            64  => 'Normal',
-            80  => 'Normal',
-            100 => 'High (or ISO 100)', #4
-            125 => '+1.0',
-            180 => 'ISO 180', #4
-            250 => '+2.0',
-            244 => '+3.0',
-            300 => 'ISO 300', #4
-            500 => 'ISO 500', #4
+        Priority => 0,
+    },
+    0x0015 => { #JD (Similar to Type2 0x2001)
+        Name => 'FirmwareDate',
+        Writable => 'string',
+        Format => 'undef', # the 'string' contains nulls
+        Count => 18,
+        PrintConv => q{
+            $_ = $val;
+            if (/^(\d{2})(\d{2})\0\0(\d{2})(\d{2})\0\0(\d{2})(.{2})\0{2}$/) {
+                my $yr = $1 + ($1 < 70 ? 2000 : 1900);
+                my $sec = $6;
+                $val = "$yr:$2:$3 $4:$5";
+                $val .= ":$sec" if $sec=~/^\d{2}$/;
+                return $val;
+            }
+            tr/\0/./;  s/\.+$//;
+            return "Unknown ($_)";
+        },
+        PrintConvInv => q{
+            $_ = $val;
+            if (/^(19|20)(\d{2}):(\d{2}):(\d{2}) (\d{2}):(\d{2})$/) {
+                return "$2$3\0\0$4$5\0\0$6\0\0\0\0";
+            } elsif (/^Unknown\s*\((.*)\)$/i) {
+                $_ = $1;  tr/./\0/;
+                return $_;
+            } else {
+                return undef;
+            }
         },
     },
     0x0016 => { #4
@@ -188,7 +209,7 @@ $VERSION = '1.19';
         },
     },
     0x0017 => { #4
-        Name => 'Filter',
+        Name => 'ColorFilter',
         Writable => 'int16u',
         PrintConv => {
             1 => 'Off',
@@ -205,6 +226,7 @@ $VERSION = '1.19';
     0x0018 => { #4
         Name => 'AFPoint',
         Writable => 'int16u',
+        Notes => 'may not be valid for all models', #JD
         PrintConv => {
             1 => 'Center',
             2 => 'Upper Left',
@@ -311,8 +333,8 @@ $VERSION = '1.19';
     },
     0x0014 => {
         Name => 'ISO',
-        Priority => 0,
         Writable => 'int16u',
+        Priority => 0,
         PrintConv => {
            3 => 50,
            4 => 64,
@@ -335,8 +357,8 @@ $VERSION = '1.19';
     0x001d => {
         Name => 'FocalLength',
         Writable => 'rational64u',
-        PrintConv => 'sprintf("%.1fmm",$val)',
-        PrintConvInv => '$val=~s/mm$//;$val',
+        PrintConv => 'sprintf("%.1f mm",$val)',
+        PrintConvInv => '$val=~s/\s*mm$//;$val',
     },
     0x001f => {
         Name => 'Saturation',
@@ -421,23 +443,49 @@ $VERSION = '1.19';
            3 => 'Shade', #3
            4 => 'Flash?',
            6 => 'Fluorescent', #3
+           9 => 'Tungsten?', #PH (EX-Z77)
            10 => 'Tungsten', #3
            12 => 'Flash',
         },
     },
+    0x2021 => { #JD (guess)
+        Name => 'AFPointPosition',
+        Writable => 'int16u',
+        Count => 4,
+        PrintConv => q{
+            my @v = split ' ', $val;
+            return 'n/a' if $v[0] == 65535 or not $v[1] or not $v[3];
+            sprintf "%.2g %.2g", $v[0]/$v[1], $v[2]/$v[3];
+        },
+    },
     0x2022 => {
         Name => 'ObjectDistance',
         Writable => 'int32u',
-        ValueConv => '$val / 1000',
-        ValueConvInv => '$val * 1000',
-        PrintConv => '"$val m"',
+        ValueConv => '$val >= 0x20000000 ? "inf" : $val / 1000',
+        ValueConvInv => '$val eq "inf" ? 0x20000000 : $val * 1000',
+        PrintConv => '$val eq "inf" ? $val : "$val m"',
         PrintConvInv => '$val=~s/\s*m$//;$val',
     },
     # 0x2023 looks interesting (values 0,1,2,3,5 in samples) - PH
+    #        - 1 for makeup mode shots (portrait?) (EX-Z450)
     0x2034 => {
         Name => 'FlashDistance',
         Writable => 'int16u',
     },
+    # 0x203e - normally 62000, but 62001 for anti-shake mode - PH
+    0x2076 => { #PH (EX-Z450)
+        # ("Enhancement" was taken already, so call this "SpecialEffect" for lack of a better name)
+        Name => 'SpecialEffectMode',
+        Writable => 'int8u',
+        Count => 3,
+        PrintConv => {
+            '0 0 0' => 'Off',
+            '1 0 0' => 'Makeup',
+            '2 0 0' => 'Mist Removal',
+            '3 0 0' => 'Vivid Landscape',
+            # have also seen '1 1 1', '2 2 4', '4 3 3', '4 4 4'
+        },
+    },
     0x3000 => {
         Name => 'RecordMode',
         Writable => 'int16u',
@@ -447,13 +495,27 @@ $VERSION = '1.19';
             4 => 'Aperture Priority', #3
             5 => 'Manual', #3
             6 => 'Best Shot', #3
+            17 => 'Movie', #PH (UHQ?)
+            19 => 'Movie (19)', #PH (HQ?, EX-P505)
+            20 => 'YouTube Movie', #PH
+            '2 0' => 'Program AE', #PH (NC)
+            '3 0' => 'Shutter Priority', #PH (NC)
+            '4 0' => 'Aperture Priority', #PH (NC)
+            '5 0' => 'Manual', #PH (NC)
+            '6 0' => 'Best Shot', #PH (NC)
         },
     },
-    # 0x3001 is ShutterMode according to ref 3!
-    0x3001 => {
-        Name => 'SelfTimer',
+    0x3001 => { #3
+        Name => 'ReleaseMode',
         Writable => 'int16u',
-        PrintConv => { 1 => 'Off' },
+        PrintConv => {
+            1 => 'Normal',
+            3 => 'AE Bracketing',
+            11 => 'WB Bracketing',
+            13 => 'Contrast Bracketing', #(not sure about translation - PH)
+            19 => 'High Speed Burst', #PH (EX-FH25, 40fps)
+            # have also seen: 2, 7(common), 14, 18 - PH
+        },
     },
     0x3002 => {
         Name => 'Quality',
@@ -468,29 +530,46 @@ $VERSION = '1.19';
         Name => 'FocusMode',
         Writable => 'int16u',
         PrintConv => {
-           0 => 'Manual?',
-           1 => 'Fixation?',
+           0 => 'Manual', #(guess at translation)
+           1 => 'Focus Lock', #(guess at translation)
            2 => 'Macro', #3
            3 => 'Single-Area Auto Focus',
+           5 => 'Infinity', #PH
            6 => 'Multi-Area Auto Focus',
         },
     },
     0x3006 => {
-        Name => 'TimeZone',
+        Name => 'HometownCity',
         Writable => 'string',
     },
     0x3007 => {
         Name => 'BestShotMode',
         Writable => 'int16u',
-        PrintConv => {
-           0 => 'Off',
-           1 => 'On?',
-        },
+        # unfortunately these numbers are model-dependent,
+        # so we can't use a lookup as usual - PH
+        PrintConv => '$val ? $val : "Off"',
+        PrintConvInv => '$val=~/(\d+)/ ? $1 : 0',
     },
     0x3008 => { #3
         Name => 'AutoISO',
         Writable => 'int16u',
-        PrintConv => { 1 => 'On', 2 => 'Off' },
+        PrintConv => {
+            1 => 'On',
+            2 => 'Off',
+            7 => 'On (high sensitivity)', #PH
+            8 => 'On (anti-shake)', #PH
+            10 => 'High Speed', #PH (EX-FC150)
+        },
+    },
+    0x3009 => { #PH (EX-Z77)
+        Name => 'AFMode',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'On',
+            # have seen 2, 3(portrait) and 5(auto mode)
+            4 => 'Face Recognition', # "Family First"
+        },
     },
     0x3011 => { #3
         Name => 'Sharpness',
@@ -508,24 +587,51 @@ $VERSION = '1.19';
         Writable => 'undef',
     },
     0x3014 => {
-        Name => 'CCDISOSensitivity',
+        Name => 'ISO',
         Writable => 'int16u',
-        Description => 'CCD ISO Sensitivity',
+        Priority => 0,
     },
     0x3015 => {
         Name => 'ColorMode',
         Writable => 'int16u',
-        PrintConv => { 0 => 'Off' },
+        PrintConv => {  
+            0 => 'Off',
+            2 => 'Black & White', #PH (EX-Z400,FH20)
+            3 => 'Sepia', #PH (EX-Z400)
+        },
     },
     0x3016 => {
         Name => 'Enhancement',
         Writable => 'int16u',
-        PrintConv => { 0 => 'Off' },
+        PrintConv => {
+            0 => 'Off',
+            1 => 'Scenery', #PH (NC) (EX-Z77)
+            3 => 'Green', #PH (EX-Z77)
+            5 => 'Underwater', #PH (NC) (EX-Z77)
+            9 => 'Flesh Tones', #PH (EX-Z77)
+        },
     },
     0x3017 => {
-        Name => 'Filter',
+        Name => 'ColorFilter',
         Writable => 'int16u',
-        PrintConv => { 0 => 'Off' },
+        PrintConv => {
+            0 => 'Off',
+            1 => 'Blue', #PH (FH20,Z400)
+            3 => 'Green', #PH (FH20)
+            4 => 'Yellow', #PH (FH20)
+            5 => 'Red', #PH (FH20,Z77)
+            6 => 'Purple', #PH (FH20,Z77,Z400)
+            7 => 'Pink', #PH (FH20)
+        },
+    },
+    0x301b => { #PH
+        Name => 'UnknownMode',
+        Writable => 'int16u',
+        Unknown => 1,
+        PrintConv => {
+            0 => 'Normal',
+            8 => 'Silent Movie',
+        },
     },
     0x301c => { #3
         Name => 'SequenceNumber', # for continuous shooting
@@ -544,10 +650,86 @@ $VERSION = '1.19';
             0 => 'Off',
             1 => 'On',
             2 => 'Best Shot',
+            # 3 observed in MOV videos (EX-V7)
+            # (newer models write 2 numbers here - PH)
+            '0 0' => 'Off', #PH
+            '16 0' => 'Slow Shutter', #PH (EX-Z77)
+            '18 0' => 'Anti-Shake', #PH (EX-Z77)
+            '20 0' => 'High Sensitivity', #PH (EX-Z77)
+            '0 3' => 'CCD Shift', #PH (guess)
+            '2 3' => 'High Speed Anti-Shake', #PH (EX-FC150)
+        },
+    },
+    0x302a => { #PH (EX-Z450)
+        Name => 'LightingMode', #(just guessing here)
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'High Dynamic Range', # (EX-Z77 anti-blur shot)
+            5 => 'Shadow Enhance Low', #(NC)
+            6 => 'Shadow Enhance High', #(NC)
+        },
+    },
+    0x302b => { #PH (EX-Z77)
+        Name => 'PortraitRefiner',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Off',
+            1 => '+1',
+            2 => '+2',
+        },
+    },
+    0x3030 => { #PH (EX-Z450)
+        Name => 'SpecialEffectLevel',
+        Writable => 'int16u',
+    },
+    0x3031 => { #PH (EX-Z450)
+        Name => 'SpecialEffectSetting',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'Makeup',
+            2 => 'Mist Removal',
+            3 => 'Vivid Landscape',
+        },
+    },
+    0x4001 => { #PH (AVI videos)
+        Name => 'CaptureFrameRate',
+        Writable => 'int16u',
+        Count => -1,
+        ValueConv => q{
+            my @v=split(" ",$val);
+            return $val / 1000 if @v == 1;
+            return $v[1] ? "$v[1]-$v[0]" : ($v[0] > 10000 ? $v[0] / 1000 : $v[0]);
+        },
+        ValueConvInv => '$val <= 60 ? $val * 1000 : int($val) . " 0"',
+    },
+    # 0x4002 - AVI videos, related to video quality or size - PH
+    0x4003 => { #PH (AVI videos)
+        Name => 'VideoQuality',
+        Writable => 'int16u',
+        PrintConv => {
+            1 => 'Standard',
+            # 2 - could this be LP?
+            3 => 'HD',
+            5 => 'Low', # used in High Speed modes
         },
     },
 );
 
+# tags in Casio AVI videos (ref PH)
+%Image::ExifTool::Casio::AVI = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    FIRST_ENTRY => 0,
+    NOTES => 'This information is found in Casio GV-10 AVI videos.',
+    0 => {
+        Name => 'Software', # (equivalent to RIFF Software tag)
+        Format => 'string',
+    },
+);
+
+
 1;  # end
 
 __END__
@@ -567,7 +749,7 @@ Casio maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -582,7 +764,8 @@ under the same terms as Perl itself.
 
 =head1 ACKNOWLEDGEMENTS
 
-Thanks to Joachim Loehr for adding support for the type 2 maker notes.
+Thanks to Joachim Loehr for adding support for the type 2 maker notes, and
+Jens Duttke for decoding some tags.
 
 =head1 SEE ALSO
 
