@@ -22,7 +22,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.17';
+$VERSION = '1.24';
 
 sub ProcessKodakIFD($$$);
 sub ProcessKodakText($$$);
@@ -256,8 +256,7 @@ sub WriteKodakIFD($$$);
     0x6b => {
         Name => 'Sharpness',
         Format => 'int8s',
-        PrintConv => 'Image::ExifTool::Exif::PrintParameter($val)',
-        PrintConvInv => '$val=~/normal/i ? 0 : $val',
+        %Image::ExifTool::Exif::printParameter,
     },
 );
 
@@ -330,8 +329,7 @@ sub WriteKodakIFD($$$);
     0x37 => {
         Name => 'Sharpness',
         Format => 'int8s',
-        PrintConv => 'Image::ExifTool::Exif::PrintParameter($val)',
-        PrintConvInv => '$val=~/normal/i ? 0 : $val',
+        %Image::ExifTool::Exif::printParameter,
     },
     0x38 => {
         Name => 'ExposureTime',
@@ -517,17 +515,18 @@ sub WriteKodakIFD($$$);
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     NOTES => q{
-        Kodak models such as the P712, P850, P880, Z612, Z712 and Z8612 use standard
-        TIFF IFD format for the maker notes.  In keeping with Kodak's strategy of
-        inconsitent makernotes, some models such as the Z1085 also use these tags,
-        but for these models the makernotes begin with a TIFF header instead of an
-        IFD entry count and use relative instead of absolute offsets.  There is a
-        large amount of information stored in these maker notes (apparently with
-        much duplication), but relatively few tags have so far been decoded.
+        Kodak models such as the ZD710, P712, P850, P880, V1233, V1253, V1275,
+        V1285, Z612, Z712, Z812, Z885 use standard TIFF IFD format for the maker
+        notes.  In keeping with Kodak's strategy of inconsistent makernotes, models
+        such as the M380, M1033, M1093, V1073, V1273, Z1012, Z1085 and Z8612
+        also use these tags, but these makernotes begin with a TIFF header instead
+        of an IFD entry count and use relative instead of absolute offsets.  There
+        is a large amount of information stored in these maker notes (apparently
+        with much duplication), but relatively few tags have so far been decoded.
     },
     0xfc00 => {
         Name => 'SubIFD0',
-        Groups => { 1 => 'MakerNotes' },    # SubIFD needs group 1 set
+        Groups => { 1 => 'MakerNotes' },        # SubIFD needs group 1 set
         Flags => 'SubIFD',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::SubIFD0',
@@ -538,7 +537,8 @@ sub WriteKodakIFD($$$);
     # the byte ordering used
     0xfc01 => {
         Name => 'SubIFD1',
-        Groups => { 1 => 'MakerNotes' },    # SubIFD needs group 1 set
+        Condition => '$$valPt ne "\0\0\0\0"',   # may be zero if dir doesn't exist
+        Groups => { 1 => 'MakerNotes' },        # SubIFD needs group 1 set
         Flags => 'SubIFD',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::SubIFD1',
@@ -548,7 +548,8 @@ sub WriteKodakIFD($$$);
     },
     0xfc02 => {
         Name => 'SubIFD2',
-        Groups => { 1 => 'MakerNotes' },    # SubIFD needs group 1 set
+        Condition => '$$valPt ne "\0\0\0\0"',   # may be zero if dir doesn't exist
+        Groups => { 1 => 'MakerNotes' },        # SubIFD needs group 1 set
         Flags => 'SubIFD',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::SubIFD2',
@@ -558,7 +559,8 @@ sub WriteKodakIFD($$$);
     },
     0xfc03 => {
         Name => 'SubIFD3',
-        Groups => { 1 => 'MakerNotes' },    # SubIFD needs group 1 set
+        Condition => '$$valPt ne "\0\0\0\0"',   # may be zero if dir doesn't exist
+        Groups => { 1 => 'MakerNotes' },        # SubIFD needs group 1 set
         Flags => 'SubIFD',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::SubIFD3',
@@ -570,7 +572,8 @@ sub WriteKodakIFD($$$);
     # in case it is used by future models -- ignored if pointer is zero)
     0xfc04 => {
         Name => 'SubIFD4',
-        Groups => { 1 => 'MakerNotes' },    # SubIFD needs group 1 set
+        Condition => '$$valPt ne "\0\0\0\0"',   # may be zero if dir doesn't exist
+        Groups => { 1 => 'MakerNotes' },        # SubIFD needs group 1 set
         Flags => 'SubIFD',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::SubIFD4',
@@ -580,7 +583,8 @@ sub WriteKodakIFD($$$);
     },
     0xfc05 => {
         Name => 'SubIFD5',
-        Groups => { 1 => 'MakerNotes' },    # SubIFD needs group 1 set
+        Condition => '$$valPt ne "\0\0\0\0"',   # may be zero if dir doesn't exist
+        Groups => { 1 => 'MakerNotes' },        # SubIFD needs group 1 set
         Flags => 'SubIFD',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::SubIFD5',
@@ -590,7 +594,8 @@ sub WriteKodakIFD($$$);
     },
     0xff00 => {
         Name => 'CameraInfo',
-        Groups => { 1 => 'MakerNotes' },    # SubIFD needs group 1 set
+        Condition => '$$valPt ne "\0\0\0\0"',   # may be zero if dir doesn't exist
+        Groups => { 1 => 'MakerNotes' },        # SubIFD needs group 1 set
         Flags => 'SubIFD',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::CameraInfo',
@@ -647,6 +652,86 @@ sub WriteKodakIFD($$$);
     },
 );
 
+# more Kodak IFD-format maker notes (ref PH)
+%Image::ExifTool::Kodak::Type10 = (
+    WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
+    CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    PRIORITY => 0,
+    NOTES => q{
+        Another variation of the IFD-format type, this time with just a byte order
+        indicator instead of a full TIFF header.  These tags are used by the Z980.
+    },
+    # 0x01 int16u - always 0
+    0x02 => {
+        Name => 'PreviewImageSize',
+        Writable => 'int16u',
+        Count => 2,
+    },
+    # 0x03 int32u - ranges from about 33940 to 40680
+    # 0x04 int32u - always 18493
+    # 0x06 undef[4] - 07 d9 04 11
+    # 0x07 undef[3] - varies
+    # 0x08 int16u - 1 (mostly), 2
+    # 0x09 int16u - 255
+    # 0x0b int16u[2] - '0 0' (mostly), '20 0', '21 0', '1 0'
+    # 0x0c int16u - 1 (mostly), 3, 259, 260
+    # 0x0d int16u - 0
+    # 0x0e int16u - 0, 1, 2 (MeteringMode? 0=Partial, 1,2=Multi)
+    # 0x0f int16u - 0, 5 (MeteringMode? 0=Multi, 5=Partial)
+    # 0x10 int16u - ranges from about 902 to 2308
+    0x12 => {
+        Name => 'ExposureTime',
+        Writable => 'int32u',
+        ValueConv => '$val / 100000',
+        ValueConvInv => '$val * 100000',
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+        PrintConvInv => 'eval $val',
+    },
+    0x13 => {
+        Name => 'FNumber',
+        Writable => 'int16u',
+        ValueConv => '$val / 10',
+        ValueConvInv => '$val * 10',
+    },
+    0x14 => {
+        Name => 'ISO',
+        Writable => 'int16u',
+        ValueConv => 'exp($val/3*log(2))*25',
+        ValueConvInv => '3*log($val/25)/log(2)',
+        PrintConv => 'int($val + 0.5)',
+        PrintConvInv => '$val',
+    },
+    # 0x15 int16u - 18-25 (SceneMode? 21=auto, 24=Aperture Priority, 19=high speed)
+    # 0x16 int16u - 50
+    # 0x17 int16u - 0, 65535 (MeteringMode? 0=Multi, 65535=Partial)
+    # 0x19 int16u - 0, 4 (WhiteBalance? 0=Auto, 4=Manual)
+    # 0x1a int16u - 0, 65535
+    # 0x1b int16u - 416-696
+    # 0x1c int16u - 251-439 (low when 0x1b is high)
+    0x1d => {
+        Name => 'FocalLength',
+        Writable => 'int32u',
+        ValueConv => '$val / 100',
+        ValueConvInv => '$val * 100',
+        PrintConv => '"$val mm"',
+        PrintConvInv => '$val=~s/\s*mm//;$val',
+    },
+    # 0x1e int16u - 100
+    # 0x1f int16u - 0, 1
+    # 0x20,0x21 int16u - 1
+    # 0x27 undef[4] - fe ff ff ff
+    # 0x32 undef[4] - 00 00 00 00
+    # 0x61 int32u[2] - '0 0' or '34050 0'
+    # 0x62 int8u - 0, 1
+    # 0x63 int8u - 1
+    # 0x64,0x65 int8u - 0, 1, 2
+    # 0x66 int32u - 0
+    # 0x67 int32u - 3
+    # 0x68 int32u - 0
+    # 0x3fe undef[2540]
+);
+
 # Kodak SubIFD0 tags (ref PH)
 %Image::ExifTool::Kodak::SubIFD0 = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
@@ -889,6 +974,101 @@ my %sceneModeUsed = (
     },
     # 0x1002 - related to focal length (1=wide, 32=full zoom)
     # 0x1006 - pictures remaining? (gradually decreases as pictures are taken)
+#
+# the following unknown Kodak tags in subIFD3 may store an IFD count of 0 or 1 instead
+# of the correct value (which changes from model to model).  This bad count is fixed
+# with the "FixCount" patch.  Models known to have this problem include:
+# M380, M1033, M1093IS, V1073, V1233, V1253, V1273, V1275, V1285, Z612, Z712,
+# Z812, Z885, Z915, Z950, Z1012IS, Z1085IS, ZD710
+#
+    0x2007 => {
+        Name => 'Kodak_SubIFD3_0x2007',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x2008 => {
+        Name => 'Kodak_SubIFD3_0x2008',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x2009 => {
+        Name => 'Kodak_SubIFD3_0x2009',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x200a => {
+        Name => 'Kodak_SubIFD3_0x200a',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x200b => {
+        Name => 'Kodak_SubIFD3_0x200b',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x3020 => {
+        Name => 'Kodak_SubIFD3_0x3020',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x3030 => {
+        Name => 'Kodak_SubIFD3_0x3030',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x3040 => {
+        Name => 'Kodak_SubIFD3_0x3040',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x3050 => {
+        Name => 'Kodak_SubIFD3_0x3050',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x3060 => {
+        Name => 'Kodak_SubIFD3_0x3060',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x8001 => {
+        Name => 'Kodak_SubIFD3_0x8001',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x8002 => {
+        Name => 'Kodak_SubIFD3_0x8002',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x8003 => {
+        Name => 'Kodak_SubIFD3_0x8003',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x8004 => {
+        Name => 'Kodak_SubIFD3_0x8004',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x8005 => {
+        Name => 'Kodak_SubIFD3_0x8005',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x8006 => {
+        Name => 'Kodak_SubIFD3_0x8006',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x8007 => {
+        Name => 'Kodak_SubIFD3_0x8007',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x8008 => {
+        Name => 'Kodak_SubIFD3_0x8008',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x8009 => {
+        Name => 'Kodak_SubIFD3_0x8009',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x800a => {
+        Name => 'Kodak_SubIFD3_0x800a',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x800b => {
+        Name => 'Kodak_SubIFD3_0x800b',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
+    0x800c => {
+        Name => 'Kodak_SubIFD3_0x800c',
+        Flags => [ 'FixCount', 'Unknown', 'Hidden' ],
+    },
 );
 
 # Kodak SubIFD4 tags (ref PH)
@@ -991,7 +1171,7 @@ my %sceneModeUsed = (
 
 # tags found in the KodakIFD (in IFD0 of KDC, DCR, TIFF and JPEG images) (ref PH)
 %Image::ExifTool::Kodak::IFD = (
-    GROUPS => { 0 => 'EXIF', 1 => 'KodakIFD', 2 => 'Image'},
+    GROUPS => { 0 => 'MakerNotes', 1 => 'KodakIFD', 2 => 'Image'},
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
     WRITE_GROUP => 'KodakIFD',
@@ -1028,17 +1208,37 @@ my %sceneModeUsed = (
             TagTable => 'Image::ExifTool::Kodak::TextualInfo',
         },
     },
-    # 0x03fc: some sort of white balance index (ref 3)
-    # 0x03fd: manual white balance information (ref 3)
+    0x03fc => { #3
+        Name => 'WhiteBalance',
+        Writable => 'int16u',
+        Priority => 0,
+        PrintConv => { },   # no values yet known
+    },
+    0x03fd => { #3
+        Name => 'Processing',
+        Condition => '$count == 72',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::Processing',
+        },
+    },
     0x0401 => {
         Name => 'Time',
         Groups => { 2 => 'Time' },
         Writable => 'string',
     },
     0x0414 => { Name => 'NCDFileInfo',      Writable => 'string' },
-    0x0846 => { Name => 'ColorTemperature', Writable => 'int16u' }, #3
-    # 0x0852: used in calculating WB levels (ref 3)
-    # 0x085c: used in calculating WB levels (ref 3)
+    0x0846 => { #3
+        Name => 'ColorTemperature',
+        Writable => 'int16u',
+    },
+    0x0852 => 'WB_RGBMul0', #3
+    0x0853 => 'WB_RGBMul1', #3
+    0x0854 => 'WB_RGBMul2', #3
+    0x0855 => 'WB_RGBMul3', #3
+    0x085c => { Name => 'WB_RGBCoeffs0', Binary => 1 }, #3
+    0x085d => { Name => 'WB_RGBCoeffs1', Binary => 1 }, #3
+    0x085e => { Name => 'WB_RGBCoeffs2', Binary => 1 }, #3
+    0x085f => { Name => 'WB_RGBCoeffs3', Binary => 1 }, #3
     # 0x090d: linear table (ref 3)
     # 0x0c81: some sort of date (manufacture date?) - PH
     0x0ce5 => { Name => 'FirmwareVersion',  Writable => 'string' },
@@ -1047,6 +1247,61 @@ my %sceneModeUsed = (
     0x1784 => { Name => 'ISO',              Writable => 'int32u' }, #3
 );
 
+# contains WB adjust set in software (ref 3)
+%Image::ExifTool::Kodak::Processing = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    FORMAT => 'int16u',
+    FIRST_ENTRY => 0,
+    20 => {
+        Name => 'WB_RGBLevels',
+        Format => 'int16u[3]',
+        ValueConv => q{
+            my @a = split ' ',$val;
+            foreach (@a) {
+                $_ = 2048 / $_ if $_;
+            }
+            return join ' ', @a;
+        }
+    },
+);
+
+# tags found in the Kodak KDC_IFD (in IFD0 of KDC images) (ref 3)
+%Image::ExifTool::Kodak::KDC_IFD = (
+    GROUPS => { 0 => 'MakerNotes', 1 => 'KDC_IFD', 2 => 'Image'},
+    WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
+    CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
+    WRITE_GROUP => 'KDC_IFD',
+    SET_GROUP1 => 1,
+    NOTES => q{
+        These tags are found in a separate IFD of KDC images from some newer Kodak
+        models such as the P880 and Z1015IS.
+    },
+    0xfa00 => {
+        Name => 'SerialNumber', #PH (unverified)
+        Writable => 'string',
+    },
+    0xfa0d => {
+        Name => 'WhiteBalance',
+        Writable => 'int8u',
+        PrintConv => { #PH
+            0 => 'Auto',
+            1 => 'Fluorescent', # (NC)
+            2 => 'Tungsten', # (NC)
+            3 => 'Daylight', # (NC)
+            6 => 'Shade', # (NC, called "Open Shade" by Kodak)
+        },
+    },
+    # the following tags are numbered for use in the Composite tag lookup
+    0xfa25 => 'WB_RGBLevelsAuto',
+    0xfa27 => 'WB_RGBLevelsTungsten', # (NC)
+    0xfa28 => 'WB_RGBLevelsFluorescent', # (NC)
+    0xfa29 => 'WB_RGBLevelsDaylight', # (NC)
+    0xfa2a => 'WB_RGBLevelsShade', # (NC)
+);
+
 # textual-based Kodak TextualInfo tags (not found in KDC images) (ref PH)
 %Image::ExifTool::Kodak::TextualInfo = (
     GROUPS => { 0 => 'MakerNotes', 1 => 'Kodak', 2 => 'Image'},
@@ -1079,6 +1334,7 @@ my %sceneModeUsed = (
             'S' => 'Shutter Priority', #(NC)
             'P' => 'Program', #(NC)
             'B' => 'Bulb', #(NC)
+            # have seen "Manual (M)" written by DCS760C - PH
         },
     },
     'Firmware Version' => 'FirmwareVersion',
@@ -1111,7 +1367,7 @@ my %sceneModeUsed = (
         Name => 'Time',
         Groups => { 2 => 'Time' },
     },
-    'White balance' => 'Whitebalance',
+    'White balance' => 'WhiteBalance',
     'Width'         => 'KodakImageWidth',
     '_other_info'   => {
         Name => 'OtherInfo',
@@ -1261,6 +1517,7 @@ my %sceneModeUsed = (
 
 # Kodak composite tags
 %Image::ExifTool::Kodak::Composite = (
+    GROUPS => { 2 => 'Camera' },
     DateCreated => {
         Groups => { 2 => 'Time' },
         Require => {
@@ -1269,11 +1526,71 @@ my %sceneModeUsed = (
         },
         ValueConv => '"$val[0]:$val[1]"',
     },
+    WB_RGBLevels => {
+        Require => {
+            0 => 'KDC_IFD:WhiteBalance',
+        },
+        # indices of the following entries are KDC_IFD:WhiteBalance + 1
+        Desire => {
+            1 => 'WB_RGBLevelsAuto',
+            2 => 'WB_RGBLevelsFluorescent',
+            3 => 'WB_RGBLevelsTungsten',
+            4 => 'WB_RGBLevelsDaylight',
+            5 => 'WB_RGBLevels4',
+            6 => 'WB_RGBLevels5',
+            7 => 'WB_RGBLevelsShade',
+        },
+        ValueConv => '$val[$val[0] + 1]',
+    },
+    WB_RGBLevels2 => {
+        Name => 'WB_RGBLevels',
+        Require => {
+            0 => 'KodakIFD:WhiteBalance',
+            1 => 'WB_RGBMul0',
+            2 => 'WB_RGBMul1',
+            3 => 'WB_RGBMul2',
+            4 => 'WB_RGBMul3',
+            5 => 'WB_RGBCoeffs0',
+            6 => 'WB_RGBCoeffs1',
+            7 => 'WB_RGBCoeffs2',
+            8 => 'WB_RGBCoeffs3',
+        },
+        # indices of the following entries are KDC_IFD:WhiteBalance + 1
+        Desire => {
+            9 => 'KodakIFD:ColorTemperature',
+            10 => 'Kodak:WB_RGBLevels',
+        },
+        ValueConv => 'Image::ExifTool::Kodak::CalculateRGBLevels(@val)',
+    },
 );
 
 # add our composite tags
 Image::ExifTool::AddCompositeTags('Image::ExifTool::Kodak');
 
+#------------------------------------------------------------------------------
+# Calculate RGB levels from associated tags (ref 3)
+# Inputs: 0) KodakIFD:WhiteBalance, 1-4) WB_RGBMul0-3, 5-8) WB_RGBCoeffs0-3
+#         9) (optional) KodakIFD:ColorTemperature, 10) (optional) Kodak:WB_RGBLevels
+# Returns: WB_RGBLevels or undef
+sub CalculateRGBLevels(@)
+{
+    return undef if $_[10]; # use existing software levels if they exist
+    my $wbi = $_[0];
+    return undef if $wbi < 0 or $wbi > 3;
+    my @mul = split ' ', $_[$wbi + 1], 13; # (only use the first 12 coeffs)
+    my @coefs = split ' ', ${$_[$wbi + 5]}; # (extra de-reference for Binary data)
+    my $wbtemp100 = ($_[9] || 6500) / 100;
+    return undef unless @mul >= 3 and @coefs >= 12;
+    my ($i, $c, $n, $num, @cam_mul);
+    for ($c=$n=0; $c<3; ++$c) {
+        for ($num=$i=0; $i<4; ++$i) {
+            $num += $coefs[$n++] * ($wbtemp100 ** $i);
+        }
+        $cam_mul[$c] = 2048 / ($num * $mul[$c]);
+    }
+    return join(' ', @cam_mul);
+}
+
 #------------------------------------------------------------------------------
 # Process Kodak textual TextualInfo
 # Inputs: 0) ExifTool object ref, 1) dirInfo hash ref, 2) tag table ref
@@ -1288,6 +1605,7 @@ sub ProcessKodakText($$$)
     $data =~ s/\0.*//s;     # truncate at null if it exists
     my @lines = split /[\n\r]+/, $data;
     my ($line, $success, @other, $tagInfo);
+    $exifTool->VerboseDir('Kodak Text');
     foreach $line (@lines) {
         if ($line =~ /(.*?):\s*(.*)/) {
             my ($tag, $val) = ($1, $2);
@@ -1303,7 +1621,7 @@ sub ProcessKodakText($$$)
                 $tagInfo = { Name => $tagName };
                 Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
             }
-            $exifTool->FoundTag($tagInfo, $val);
+            $exifTool->HandleTag($tagTablePtr, $tag, $val, TagInfo => $tagInfo);
             $success = 1;
         } else {
             # strip off leading/trailing white space and ignore blank lines
@@ -1336,7 +1654,7 @@ sub ProcessKodakIFD($$$)
     $$dirInfo{DirLen} -= 2;
     if ($exifTool->{HTML_DUMP}) {
         my $base = $$dirInfo{Base} + $$dirInfo{DataPos};
-        $exifTool->HtmlDump($dirStart+$base, 2, "Byte Order Mark");
+        $exifTool->HDump($dirStart+$base, 2, "Byte Order Mark");
     }
     return Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
 }
@@ -1384,7 +1702,7 @@ interpret Kodak maker notes EXIF meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
