@@ -18,15 +18,16 @@
 #               3) Matt Mueller private communication (tests with PS CS2)
 #               4) http://www.fileformat.info/format/psd/egff.htm
 #               5) http://www.telegraphics.com.au/svn/psdparse/trunk/resources.c
+#               6) http://libpsd.graphest.com/files/Photoshop%20File%20Formats.pdf
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Photoshop;
 
 use strict;
-use vars qw($VERSION $AUTOLOAD);
-use Image::ExifTool qw(:DataAccess);
+use vars qw($VERSION $AUTOLOAD $iptcDigestInfo);
+use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.36';
+$VERSION = '1.40';
 
 sub ProcessPhotoshop($$$);
 sub WritePhotoshop($$$);
@@ -146,7 +147,7 @@ my %psdMap = (
     0x0411 => { Unknown => 1, Name => 'ICC_Untagged' },
     0x0412 => { Unknown => 1, Name => 'EffectsVisible' },
     0x0413 => { Unknown => 1, Name => 'SpotHalftone' },
-    0x0414 => { Unknown => 1, Name => 'IDsBaseValue', Description => "ID's Base Value" },
+    0x0414 => { Unknown => 1, Name => 'IDsBaseValue', Description => 'IDs Base Value' },
     0x0415 => { Unknown => 1, Name => 'UnicodeAlphaNames' },
     0x0416 => { Unknown => 1, Name => 'IndexedColourTableCount' },
     0x0417 => { Unknown => 1, Name => 'TransparentIndex' },
@@ -182,19 +183,23 @@ my %psdMap = (
         Writable => 'string',
         Protected => 1,
         Notes => q{
-            when writing, a special value of "new" represents the new IPTC digest of the
-            output file, or deletes this tag if no IPTC exists
+            when writing, special values of "new" and "old" represent the digests of the
+            IPTC from the output and original files respectively, and are undefined if
+            the IPTC does not exist in the respective file
         },
         # also note the 'new' feature requires that the IPTC comes before this tag is written
         ValueConv => 'unpack("H*", $val)',
         ValueConvInv => q{
-            if (lc($val) eq 'new') {
-                return 'new' if eval 'require Digest::MD5';
+            if (lc($val) eq 'new' or lc($val) eq 'old') {
+                {
+                    local $SIG{'__WARN__'} = sub { };
+                    return lc($val) if eval 'require Digest::MD5';
+                }
                 warn "Digest::MD5 must be installed\n";
                 return undef;
             }
             return pack('H*', $val) if $val =~ /^[0-9a-f]{32}$/i;
-            warn "Value must be 32 hexadecimal digits or 'new'\n";
+            warn "Value must be 'new', 'old' or 32 hexadecimal digits\n";
             return undef;
         }
     },
@@ -204,7 +209,16 @@ my %psdMap = (
     0x042a => { Unknown => 1, Name => 'AlternateDuotoneColors' }, #5
     0x042b => { Unknown => 1, Name => 'AlternateSpotColors' }, #5
     # 0x07d0-0x0bb6 Path information
-    0x0bb7 => { Unknown => 1, Name => 'ClippingPathName' },
+    0x0bb7 => {
+        Name => 'ClippingPathName',
+        # convert from a Pascal string
+        ValueConv => q{
+            my $len = ord($val);
+            $val = substr($val, 1, $len) if $len < length($val);
+            return $val;
+        },
+        # there are 6 bytes of unknown information that follow the name
+    },
     0x2710 => { Unknown => 1, Name => 'PrintFlagsInfo' },
 );
 
@@ -312,6 +326,10 @@ my %psdMap = (
     GROUPS => { 2 => 'Unknown' },
 );
 
+# define reference to IPTCDigest tagInfo hash for convenience
+$iptcDigestInfo = $Image::ExifTool::Photoshop::Main{0x0425};
+
+
 #------------------------------------------------------------------------------
 # AutoLoad our writer routines when necessary
 #
@@ -369,7 +387,7 @@ sub ProcessPhotoshop($$$)
         if ($type eq '8BIM') {
             $ttPtr = $tagTablePtr;
         } elsif ($type =~ /^(PHUT|DCSR|AgHg)$/) {
-            $ttPtr = Image::ExifTool::GetTagTable('Image::ExifTool::Photoshop::Unknown');
+            $ttPtr = GetTagTable('Image::ExifTool::Photoshop::Unknown');
         } else {
             $type =~ s/([^\w])/sprintf("\\x%.2x",ord($1))/ge;
             $exifTool->Warn(qq{Bad Photoshop IRB resource "$type"});
@@ -433,13 +451,13 @@ sub ProcessPSD($$)
     my ($data, $err, $tagTablePtr);
 
     $raf->Read($data, 30) == 30 or return 0;
-    $data =~ /^8BPS\0\x01/ or return 0;
+    $data =~ /^8BPS\0([\x01\x02])/ or return 0;
     SetByteOrder('MM');
-    $exifTool->SetFileType();    # set the FileType tag
+    $exifTool->SetFileType($1 eq "\x01" ? 'PSD' : 'PSB'); # set the FileType tag
     my %dirInfo = (
         DataPt => \$data,
         DirStart => 0,
-        DirName => 'PSD',
+        DirName => 'Photoshop',
     );
     my $len = Get32u(\$data, 26);
     if ($outfile) {
@@ -450,7 +468,7 @@ sub ProcessPSD($$)
         $exifTool->InitWriteDirs(\%psdMap);
     } else {
         # process the header
-        $tagTablePtr = Image::ExifTool::GetTagTable('Image::ExifTool::Photoshop::Header');
+        $tagTablePtr = GetTagTable('Image::ExifTool::Photoshop::Header');
         $dirInfo{DirLen} = 30;
         $exifTool->ProcessDirectory(\%dirInfo, $tagTablePtr);
         $raf->Seek($len, 1) or $err = 1;    # skip over color mode data
@@ -458,7 +476,7 @@ sub ProcessPSD($$)
     $raf->Read($data, 4) == 4 or $err = 1;
     $len = Get32u(\$data, 0);
     $raf->Read($data, $len) == $len or $err = 1;
-    $tagTablePtr = Image::ExifTool::GetTagTable('Image::ExifTool::Photoshop::Main');
+    $tagTablePtr = GetTagTable('Image::ExifTool::Photoshop::Main');
     $dirInfo{DirLen} = $len;
     my $rtnVal = 1;
     if ($outfile) {
@@ -547,7 +565,7 @@ be preserved when copying Photoshop information via user-defined tags.
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -562,6 +580,8 @@ under the same terms as Perl itself.
 
 =item L<http://www.fileformat.info/format/psd/egff.htm>
 
+=item L<http://libpsd.graphest.com/files/Photoshop%20File%20Formats.pdf>
+
 =back
 
 =head1 SEE ALSO
