@@ -141,7 +141,7 @@ sub WritePSDirectory($$$$$)
         $dirInfo{DataLen} = $dirInfo{DirLen} = length $xmp;
         $dirInfo{DataPt} = \$xmp;
     }
-    my $tagTablePtr = GetTagTable("Image::ExifTool::${dirName}::Main");
+    my $tagTablePtr = Image::ExifTool::GetTagTable("Image::ExifTool::${dirName}::Main");
     my $val = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr);
     if (defined $val) {
         $dataPt = \$val;    # use modified directory
@@ -287,13 +287,10 @@ sub WriteNewTags($$$)
 
     foreach $tag (sort keys %$newTags) {
         my $tagInfo = $$newTags{$tag};
-        my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
-        next unless Image::ExifTool::IsCreating($newValueHash);
-        my $val = Image::ExifTool::GetNewValues($newValueHash);
-        if ($exifTool->Options('Verbose') > 1) {
-            my $out = $exifTool->Options('TextOut');
-            print $out "    + PostScript:$$tagInfo{Name} = '$val'\n";
-        }
+        my $nvHash = $exifTool->GetNewValueHash($tagInfo);
+        next unless Image::ExifTool::IsCreating($nvHash);
+        my $val = Image::ExifTool::GetNewValues($nvHash);
+        $exifTool->VerboseValue("+ PostScript:$$tagInfo{Name}", $val);
         Write($outfile, EncodeTag($tag, $val)) or $success = 0;
         ++$exifTool->{CHANGED};
     }
@@ -438,8 +435,8 @@ sub WritePS($$)
 #
 # rewrite PostScript data
 #
-    my $oldsep = SetInputRecordSeparator($raf);
-    unless ($oldsep and $raf->ReadLine($buff)) {
+    local $/ = GetInputRecordSeparator($raf);
+    unless ($/ and $raf->ReadLine($buff)) {
         $exifTool->Error('Invalid PostScript data');
         return 1;
     }
@@ -461,9 +458,16 @@ sub WritePS($$)
     my $editDirs = $exifTool->{EDIT_DIRS};
     my %doneDir;
 
+    # disable writing XMP to PS-format Adobe Illustrator files (it confuses Illustrator)
+    if ($$editDirs{XMP} and $$exifTool{FILE_EXT} and $$exifTool{FILE_EXT} eq 'AI') {
+        $exifTool->Warn("Can't write XMP to PS-format AI files", 1);
+        delete $$addDirs{XMP};
+        delete $$editDirs{XMP}; # (need this too or XMP will still be added)
+    }
+
     # set XMP hint flag (1 for adding, 0 for deleting, undef for no change)
     $xmpHint = 1 if $$addDirs{XMP};
-    $xmpHint = 0 if $exifTool->{DEL_GROUP}->{XMP};
+    $xmpHint = 0 if $$exifTool{DEL_GROUP}{XMP};
     $$newTags{XMP_HINT} = $xmpHint if $xmpHint;  # add special tag to newTags list
 
     my @lines;
@@ -546,13 +550,13 @@ sub WritePS($$)
                 # decode comment string (reading continuation lines if necessary)
                 $val = DecodeComment($val, $raf, \@lines, \$data);
                 $val = join $exifTool->Options('ListSep'), @$val if ref $val eq 'ARRAY';
-                my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
-                if (Image::ExifTool::IsOverwriting($newValueHash, $val)) {
-                    $verbose > 1 and print $out "    - PostScript:$$tagInfo{Name} = '$val'\n";
-                    $val = Image::ExifTool::GetNewValues($newValueHash);
+                my $nvHash = $exifTool->GetNewValueHash($tagInfo);
+                if (Image::ExifTool::IsOverwriting($nvHash, $val)) {
+                    $exifTool->VerboseValue("- PostScript:$$tagInfo{Name}", $val);
+                    $val = Image::ExifTool::GetNewValues($nvHash);
                     ++$exifTool->{CHANGED};
                     next unless defined $val;   # next if tag is being deleted
-                    $verbose > 1 and print $out "    + PostScript:$$tagInfo{Name} = '$val'\n";
+                    $exifTool->VerboseValue("+ PostScript:$$tagInfo{Name}", $val);
                     $data = EncodeTag($tag, $val);
                 }
             }
@@ -656,7 +660,8 @@ sub WritePS($$)
         delete $$newTags{XMP_HINT};
         push @notDone, 'PostScript' if %$newTags;
         foreach $dir (qw{Photoshop ICC_Profile XMP}) {
-            push @notDone, $dir if $$editDirs{$dir} and not $doneDir{$dir};
+            push @notDone, $dir if $$editDirs{$dir} and not $doneDir{$dir} and
+                                   not $$exifTool{DEL_GROUP}{$dir};
         }
         @notDone and $exifTool->Warn("Couldn't write ".join('/',@notDone).' information');
     }
@@ -724,7 +729,7 @@ Thanks to Tim Kordick for his help testing the EPS writer.
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
